@@ -7,13 +7,14 @@ use std::{
     f64,
     ops::{Range, Sub},
     sync::{
-        atomic::{AtomicBool, AtomicU32, Ordering},
+        atomic::{AtomicBool, AtomicU64, AtomicU8, Ordering},
         Arc, Mutex,
     },
     thread,
+    time::{Duration, Instant},
 };
 
-use crate::util::{poll,Polling};
+use crate::util::{poll, update_execution_position, Polling};
 
 /// Cooling schedule for simulated annealing.
 #[derive(Clone, Copy)]
@@ -23,7 +24,7 @@ pub enum CoolingSchedule {
     Fast,
 }
 impl CoolingSchedule {
-    fn decay(&self, t_start: f64, t_current: f64, step: u32) -> f64 {
+    fn decay(&self, t_start: f64, t_current: f64, step: u64) -> f64 {
         match self {
             Self::Logarithmic => t_current * (2f64).ln() / ((step + 1) as f64).ln(),
             Self::Exponential(x) => x * t_current,
@@ -32,11 +33,11 @@ impl CoolingSchedule {
     }
     // Given temperature start and temperature min, gives number of steps of decay which will occur
     //  before temperature start decays to be less than temperature min, then causing program to exit.
-    fn steps(&self, t_start: f64, t_min: f64) -> u32 {
+    fn steps(&self, t_start: f64, t_min: f64) -> u64 {
         match self {
-            Self::Logarithmic => (((2f64).ln() * t_start / t_min).exp() - 1f64).ceil() as u32,
-            Self::Exponential(x) => ((t_min / t_start).log(*x)).ceil() as u32,
-            Self::Fast => (t_start / t_min).ceil() as u32,
+            Self::Logarithmic => (((2f64).ln() * t_start / t_min).exp() - 1f64).ceil() as u64,
+            Self::Exponential(x) => ((t_min / t_start).log(*x)).ceil() as u64,
+            Self::Fast => (t_start / t_min).ceil() as u64,
         }
     }
 }
@@ -91,10 +92,10 @@ pub fn simulated_annealing<
     starting_temperature: f64,
     minimum_temperature: f64,
     cooling_schedule: CoolingSchedule,
-    samples_per_temperature: u32,
+    samples_per_temperature: u64,
     variance: f64,
 ) -> [T; N] {
-    let cpus = num_cpus::get() as u32;
+    let cpus = num_cpus::get() as u64;
     let search_cpus = cpus - 1; // 1 cpu is used for polling, this one.
 
     let steps = cooling_schedule.steps(starting_temperature, minimum_temperature);
@@ -105,12 +106,20 @@ pub fn simulated_annealing<
     let per = samples_per_temperature / search_cpus;
 
     let (best_value, best_params) = search(
+        // Generics
         ranges_arc.clone(),
         f,
         evaluation_data.clone(),
-        Arc::new(AtomicU32::new(Default::default())),
+        Arc::new(AtomicU64::new(Default::default())),
         Arc::new(Mutex::new(Default::default())),
         Arc::new(AtomicBool::new(false)),
+        Arc::new(AtomicU8::new(0)),
+        Arc::new([
+            Mutex::new((Duration::new(0, 0), 0)),
+            Mutex::new((Duration::new(0, 0), 0)),
+            Mutex::new((Duration::new(0, 0), 0)),
+        ]),
+        // Specifics
         starting_temperature,
         minimum_temperature,
         cooling_schedule,
@@ -118,25 +127,37 @@ pub fn simulated_annealing<
         variance,
     );
 
-    let (handles, links): (Vec<_>, Vec<(Arc<AtomicU32>, Arc<Mutex<f64>>)>) = (0..search_cpus)
+    let (handles, links): (Vec<_>, Vec<_>) = (0..search_cpus)
         .map(|_| {
             let ranges_clone = ranges_arc.clone();
-            let counter = Arc::new(AtomicU32::new(0));
+            let counter = Arc::new(AtomicU64::new(0));
             let thread_best = Arc::new(Mutex::new(f64::MAX));
+            let thread_execution_position = Arc::new(AtomicU8::new(0));
+            let thread_execution_time = Arc::new([
+                Mutex::new((Duration::new(0, 0), 0)),
+                Mutex::new((Duration::new(0, 0), 0)),
+                Mutex::new((Duration::new(0, 0), 0)),
+            ]);
 
             let counter_clone = counter.clone();
             let thread_best_clone = thread_best.clone();
             let thread_exit_clone = thread_exit.clone();
             let evaluation_data_clone = evaluation_data.clone();
+            let thread_execution_position_clone = thread_execution_position.clone();
+            let thread_execution_time_clone = thread_execution_time.clone();
             (
                 thread::spawn(move || {
                     search(
+                        // Generics
                         ranges_clone,
                         f,
                         evaluation_data_clone,
                         counter_clone,
                         thread_best_clone,
                         thread_exit_clone,
+                        thread_execution_position_clone,
+                        thread_execution_time_clone,
+                        // Specifics
                         starting_temperature,
                         minimum_temperature,
                         cooling_schedule,
@@ -144,12 +165,19 @@ pub fn simulated_annealing<
                         variance,
                     )
                 }),
-                (counter, thread_best),
+                (
+                    counter,
+                    (
+                        thread_best,
+                        (thread_execution_position, thread_execution_time),
+                    ),
+                ),
             )
         })
         .unzip();
-    let (counters, thread_bests): (Vec<Arc<AtomicU32>>, Vec<Arc<Mutex<f64>>>) =
-        links.into_iter().unzip();
+    let (counters, links): (Vec<Arc<AtomicU64>>, Vec<_>) = links.into_iter().unzip();
+    let (thread_bests, links): (Vec<Arc<Mutex<f64>>>, Vec<_>) = links.into_iter().unzip();
+    let (thread_execution_positions, thread_execution_times) = links.into_iter().unzip();
 
     if let Some(poll_data) = polling {
         poll(
@@ -159,6 +187,8 @@ pub fn simulated_annealing<
             steps * samples_per_temperature,
             thread_bests,
             thread_exit,
+            thread_execution_positions,
+            thread_execution_times,
         );
     }
 
@@ -194,16 +224,19 @@ pub fn simulated_annealing<
         ranges: Arc<[Range<T>; N]>,
         f: fn(&[T; N], Option<Arc<A>>) -> f64,
         evaluation_data: Option<Arc<A>>,
-        counter: Arc<AtomicU32>,
+        counter: Arc<AtomicU64>,
         best: Arc<Mutex<f64>>,
         thread_exit: Arc<AtomicBool>,
+        thread_execution_position: Arc<AtomicU8>,
+        thread_execution_times: Arc<[Mutex<(Duration, u64)>; 3]>,
         // Specific
         starting_temperature: f64,
         minimum_temperature: f64,
         cooling_schedule: CoolingSchedule,
-        samples_per_temperature: u32,
+        samples_per_temperature: u64,
         variance: f64,
     ) -> (f64, [T; N]) {
+        let mut execution_position_timer = Instant::now();
         let mut rng = thread_rng();
         // Get initial point
         let mut current_point = [Default::default(); N];
@@ -230,6 +263,7 @@ pub fn simulated_annealing<
 
         let mut step = 1;
         let mut temperature = starting_temperature;
+        // Iterate while starting temperature has yet to cool to the minimum temperature.
         while temperature >= minimum_temperature {
             // Distributions to sample from at this temperature.
             // `Normal::new(1.,1.).unwrap()` just replacement for `default()` since it doesn't implement trait.
@@ -241,24 +275,49 @@ pub fn simulated_annealing<
             ) {
                 *distribution = Normal::new(point.to_f64().unwrap(), *variance).unwrap()
             }
-
+            // Iterate over samples from this temperature
             for _ in 0..samples_per_temperature {
+                // Update execution position
+                execution_position_timer = update_execution_position(
+                    1,
+                    execution_position_timer,
+                    &thread_execution_position,
+                    &thread_execution_times,
+                );
+
                 // Samples new point
                 let mut point = [Default::default(); N];
                 for (p, r, d) in izip!(point.iter_mut(), float_ranges.iter(), distributions.iter())
                 {
                     *p = sample_normal(r, d, &mut rng);
                 }
+
+                // Update execution position
+                execution_position_timer = update_execution_position(
+                    2,
+                    execution_position_timer,
+                    &thread_execution_position,
+                    &thread_execution_times,
+                );
+
+                // Evaluates new point
                 let value = f(&point, evaluation_data.clone());
+
+                // Update execution position
+                execution_position_timer = update_execution_position(
+                    3,
+                    execution_position_timer,
+                    &thread_execution_position,
+                    &thread_execution_times,
+                );
+                // Increment counter
                 counter.fetch_add(1, Ordering::SeqCst);
 
-                let difference = value - current_value;
-
-                let allow_change = (difference / temperature).exp();
-
                 // Update:
                 // - if there is any progression
                 // - the regression `allow_change` is within a limit `rng.gen_range(0f64..1f64)`
+                let difference = value - current_value;
+                let allow_change = (difference / temperature).exp();
                 if difference < 0. || allow_change < rng.gen_range(0f64..1f64) {
                     current_point = point;
                     current_value = value;
@@ -276,6 +335,9 @@ pub fn simulated_annealing<
             step += 1;
             temperature = cooling_schedule.decay(starting_temperature, temperature, step);
         }
+        // Update execution position
+        // 0 represents ended state
+        thread_execution_position.store(0, Ordering::SeqCst);
         return (best_value, best_point);
     }
 
